==> spoofer-0.7.1.tar.gz: spoofer-0.7.1/spoof.h <==
/****************************************************************************
   Program:     $Id: spoof.h 497 2009-03-08 17:57:15Z rbeverly $
   Author:      Rob Beverly <rbeverly@mit.edu>
   Date:        $Date: 2009-03-08 13:57:15 -0400 (Sun, 08 Mar 2009) $
   Description: IP spoofer tester headers
****************************************************************************/

#if HAVE_CONFIG_H
  #include "config.h"
#endif

#ifndef _SPOOFER_H_
#define _SPOOFER_H_ 1

#ifdef _WIN32
  #pragma pack(1)
  #include <winsock2.h>
  #include <ws2tcpip.h>
  #include "/usr/include/netinet/udp.h"
  #include <unistd.h>
  #include <stdint.h>
  #undef HAVE_NETINET_IP_H
  #undef HAVE_IN_ADDR_T
#else
  #include <sys/types.h>
  #define  __FAVOR_BSD
  #include <unistd.h>
  #include <sys/socket.h>
  #include <netinet/in.h>
  #include <arpa/inet.h>
  #include <netinet/in_systm.h>
  #include <netinet/ip.h>
  #include <netinet/udp.h>
  #include <netdb.h>
#endif

#ifdef HAVE_NETINET_IP_VAR_H
  #include <netinet/ip_var.h>
#endif
#ifdef HAVE_NETINET_UDP_VAR_H
  #include <netinet/udp_var.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>
#include <errno.h>
#include "messages.h"

#define VERBOSE 0
#define DEBUG 0

#define BUFSIZE 1500
#define SMALLBUF 80
#define BIGBUF 9000
#define MAXMSGSIZE 4096
#define LISTENQ 1024

#define TTL_OUT 64
#define SRC_PORT 5353
#define CLIENT_TIMEOUT 30

#define DUPLICATES 5					/* dups for each probe */

#define SERVER "18.26.0.235"

#if VERSION != 7
  #define SERV_PORT 5353
  #define REPORT_PORT 8080
  #define ICMP_DIVERT_PORT 3002 
  #define UDP_DIVERT_PORT 3003 
  #define REPORT_HOST "spoofer-test.XXX.XXX"
#else
  #define SERV_PORT 53
  #define REPORT_PORT 80
  #define ICMP_DIVERT_PORT 2002 
  #define UDP_DIVERT_PORT 2003 
  #define REPORT_HOST "spoofer.csail.mit.edu"
#endif

#ifndef FALSE
 #define FALSE 0
 #define TRUE !FALSE
#endif

#define SENDTO_ERROR -2
#define WINRAWSOCK_ERROR -3
#ifndef SOCKET_ERROR
 #define SOCKET_ERROR -4
#endif

#ifndef LITTLE_ENDIAN
  #define LITTLE_ENDIAN   1234
  #define BIG_ENDIAN      4321
#endif
#ifndef BYTE_ORDER
  #if HAVE_BIG_ENDIAN
    #define BYTE_ORDER    BIG_ENDIAN
  #else
    #define BYTE_ORDER    LITTLE_ENDIAN
  #endif
#endif

#ifndef HAVE_NETINET_IP_H
/*
 * Structure of an internet header, naked of options.
 */
struct ip {
#ifdef _IP_VHL
        u_char  ip_vhl;                 /* version << 4 | header length >> 2 */
#else
#if BYTE_ORDER == LITTLE_ENDIAN
        u_int   ip_hl:4,                /* header length */
                ip_v:4;                 /* version */
#endif
#if BYTE_ORDER == BIG_ENDIAN
        u_int   ip_v:4,                 /* version */
                ip_hl:4;                /* header length */
#endif
#endif /* not _IP_VHL */
        u_char  ip_tos;                 /* type of service */
        u_short ip_len;                 /* total length */
        u_short ip_id;                  /* identification */
        u_short ip_off;                 /* fragment offset field */
#define IP_RF 0x8000                    /* reserved fragment flag */
#define IP_DF 0x4000                    /* dont fragment flag */
#define IP_MF 0x2000                    /* more fragments flag */
#define IP_OFFMASK 0x1fff               /* mask for fragmenting bits */
        u_char  ip_ttl;                 /* time to live */
        u_char  ip_p;                   /* protocol */
        u_short ip_sum;                 /* checksum */
        struct  in_addr ip_src,ip_dst;  /* source and dest address */
};
#endif /* HAVE_NETINET_IP_H */


#ifndef HAVE_NETINET_IP_VAR_H
/*
 * Overlay for ip header used by other protocols (tcp, udp).
 */
struct ipovly {
    u_char  ih_x1[9];       /* (unused) */
    u_char  ih_pr;          /* protocol */
    u_short ih_len;         /* protocol length */
    struct  in_addr ih_src;     /* source internet address */
    struct  in_addr ih_dst;     /* destination internet address */
};
#endif /* HAVE_NETINET_IP_VAR_H */

#ifndef HAVE_NETINET_UDP_VAR_H
/* 
 * UDP kernel structures and variables.
 */
struct  udpiphdr {
    struct  ipovly ui_i;        /* overlaid ip structure */
    struct  udphdr ui_u;        /* udp header */
};
#define ui_x1       ui_i.ih_x1
#define ui_pr       ui_i.ih_pr
#define ui_len      ui_i.ih_len
#define ui_src      ui_i.ih_src
#define ui_dst      ui_i.ih_dst
#define ui_sport    ui_u.uh_sport
#define ui_dport    ui_u.uh_dport
#define ui_ulen     ui_u.uh_ulen
#define ui_sum      ui_u.uh_sum

#endif /* HAVE_NETINET_UDP_VAR_H */

#ifndef HAVE_IN_ADDR_T
 typedef unsigned int in_addr_t;
#endif

#ifndef IPVERSION
 #define IPVERSION 4
#endif

#ifndef IP_DF
 #define IP_DF 0x4000
#endif

/* prevent windows error messages from closing DOS window */
#define winpause(x...) do { fprintf(stdout,x); \
                            fflush(stdout); \
                            while ( getchar() != '\n'); \
                          } while (0)
#define winfatal(x...) do { fprintf(stdout,x); \
							fprintf(stdout,"\nPress any key to continue.\n"); \
                            fflush(stdout); \
                            while ( getchar() != '\n'); \
							exit(-1); \
                          } while (0)

/* prettyprint */
#define NIPQUAD(addr) \
  ((unsigned char *)&addr)[0], \
  ((unsigned char *)&addr)[1], \
  ((unsigned char *)&addr)[2], \
  ((unsigned char *)&addr)[3]

/* error messages */
#define WIN_SEND_ERR "** This Windows system blocks raw sockets; reverting to raw ethernet.\n"
#define WINFAIL 999
#define func() do {if (DEBUG) fprintf(stdout,"[%s()][file: %s][line: %d]\n",__FUNCTION__,__FILE__,__LINE__); } while (0)
#define warn(x...) do {fprintf(stdout,"*** Warn: "); fprintf(stdout,x); fprintf(stdout,"\n");} while (0)
#define debug(level,x...) do {if (verbose >= level) {fprintf(stdout,x);} } while (0)

typedef void Sigfunc(int);

/* precasts: net.c */
int craftPacket(unsigned char *, unsigned char *, int, struct sockaddr_in *, struct sockaddr_in *, u_char);
void craftReport(unsigned char *, unsigned short, test_t *);
unsigned short in_cksum(unsigned short *, int);
void genSequence(unsigned char *, int);
Sigfunc *signal(int signo, Sigfunc *func);
Sigfunc *Signal(int signo, Sigfunc *func);
void sig_chld(int signo);
ssize_t Readn(int fd, void *ptr, size_t nbytes, int wait);
void Writen(int fd, void *ptr, size_t nbytes);
ssize_t Readline(int fd, void *ptr, size_t maxlen);
ssize_t readn(int fd, void *vptr, size_t n, int wait);
int checkRFC1918(struct in_addr *in);
int newSocket(int, int);
int connectSock(int, struct sockaddr_in *, int);
int optSocketTTL(int, int);
int optSocketHDRINCL(int);
int socketSend(int, void *, int, struct sockaddr_in *, int);
int socketSendSpoof(int, void *, int, struct sockaddr_in *, int);

/* precasts: util.c */
unsigned long byte32Swap(unsigned long);
unsigned short byte16Swap(unsigned short);
void binDump(unsigned char *, int);
void checkroot();
void confirm();
void win_init();
void banner();
void spoofSleep();
void fatal(char *, ...);
void pfatal(char *);
void streamSetup();
char *timestamp();

/* precasts: trace.c */
int findtr(char *traceroute);
int runtrace(struct in_addr *dest, char *result, int numdest);

/* precasts: dnetspoof.c */
int etherSend();
int registerEther();
int deregisterEther();
int findInterfaces();
void selectInterface();

/* precasts: messages.c */
char *type(uint8_t typ);
void printAddressList(struct in_addr *iplist, int num);
void printTest(test_t *test, int);
void printTestList(unsigned char *msg, int);
void printMsg(unsigned char *, int);
int readMsg(int sock, spoof_msg_t **message, enum msg_types);
void defaultMsg(spoof_msg_t *msg);
int errorRequest(unsigned char *);
int helloRequest(unsigned char *);
int reportRequest(unsigned char *msg, unsigned char *seqno, unsigned long src_addr, unsigned long dst_addr, int nat);
int doneRequest(unsigned char *msg);
int probeRequest(unsigned char *msg);
int readProbeList(unsigned char *buf, test_t **testlistptr);
int traceFilterRequest(unsigned char *msg);

#endif

==> spoofer-0.7.3.tar.gz: spoofer-0.7.3/spoof.h <==
/****************************************************************************
   Program:     $Id: spoof.h 540 2009-03-27 17:05:09Z rbeverly $
   Author:      Rob Beverly <rbeverly@mit.edu>
   Date:        $Date: 2009-03-27 13:05:09 -0400 (Fri, 27 Mar 2009) $
   Description: IP spoofer tester headers
****************************************************************************/

#if HAVE_CONFIG_H
  #include "config.h"
#endif

#ifndef _SPOOFER_H_
#define _SPOOFER_H_ 1

#ifdef _WIN32
  #pragma pack(1)
  #include <winsock2.h>
  #include <ws2tcpip.h>
  #include "/usr/include/netinet/udp.h"
  #include <unistd.h>
  #include <stdint.h>
  #undef HAVE_NETINET_IP_H
  #undef HAVE_IN_ADDR_T
#else
  #include <sys/types.h>
  #define  __FAVOR_BSD
  #include <unistd.h>
  #include <sys/socket.h>
  #include <netinet/in.h>
  #include <arpa/inet.h>
  #include <netinet/in_systm.h>
  #include <netinet/ip.h>
  #include <netinet/udp.h>
  #include <netdb.h>
#endif

#ifdef HAVE_NETINET_IP_VAR_H
  #include <netinet/ip_var.h>
#endif
#ifdef HAVE_NETINET_UDP_VAR_H
  #include <netinet/udp_var.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>
#include <errno.h>
#include "messages.h"

#define VERBOSE 0
#define DEBUG 0
//#define REGRESS 1
#ifdef REGRESS
#undef VERSION
#define VERSION 99
#endif

#define BUFSIZE 1500
#define SMALLBUF 80
#define BIGBUF 9000
#define MAXMSGSIZE 4096
#define LISTENQ 1024

#define TTL_OUT 64
#define SRC_PORT 5353
#define CLIENT_TIMEOUT 30
#define MAX_TRACEROUTE_CHILD 10

#define DUPLICATES 5					/* dups for each probe */

#define SERVER "18.26.0.235"

#if VERSION != 7
  #define SERV_PORT 5353
  #define REPORT_PORT 8080
  #define ICMP_DIVERT_PORT 3002 
  #define UDP_DIVERT_PORT 3003 
  #define REPORT_HOST "spoofer-test.XXX.XXX"
#else
  #define SERV_PORT 53
  #define REPORT_PORT 80
  #define ICMP_DIVERT_PORT 2002 
  #define UDP_DIVERT_PORT 2003 
  #define REPORT_HOST "spoofer.csail.mit.edu"
#endif

#ifndef FALSE
 #define FALSE 0
 #define TRUE !FALSE
#endif

#define SENDTO_ERROR -2
#define WINRAWSOCK_ERROR -3
#ifndef SOCKET_ERROR
 #define SOCKET_ERROR -4
#endif

#ifndef LITTLE_ENDIAN
  #define LITTLE_ENDIAN   1234
  #define BIG_ENDIAN      4321
#endif
#ifndef BYTE_ORDER
  #if HAVE_BIG_ENDIAN
    #define BYTE_ORDER    BIG_ENDIAN
  #else
    #define BYTE_ORDER    LITTLE_ENDIAN
  #endif
#endif

#ifndef HAVE_NETINET_IP_H
/*
 * Structure of an internet header, naked of options.
 */
struct ip {
#ifdef _IP_VHL
        u_char  ip_vhl;                 /* version << 4 | header length >> 2 */
#else
#if BYTE_ORDER == LITTLE_ENDIAN
        u_int   ip_hl:4,                /* header length */
                ip_v:4;                 /* version */
#endif
#if BYTE_ORDER == BIG_ENDIAN
        u_int   ip_v:4,                 /* version */
                ip_hl:4;                /* header length */
#endif
#endif /* not _IP_VHL */
        u_char  ip_tos;                 /* type of service */
        u_short ip_len;                 /* total length */
        u_short ip_id;                  /* identification */
        u_short ip_off;                 /* fragment offset field */
#define IP_RF 0x8000                    /* reserved fragment flag */
#define IP_DF 0x4000                    /* dont fragment flag */
#define IP_MF 0x2000                    /* more fragments flag */
#define IP_OFFMASK 0x1fff               /* mask for fragmenting bits */
        u_char  ip_ttl;                 /* time to live */
        u_char  ip_p;                   /* protocol */
        u_short ip_sum;                 /* checksum */
        struct  in_addr ip_src,ip_dst;  /* source and dest address */
};
#endif /* HAVE_NETINET_IP_H */


#ifndef HAVE_NETINET_IP_VAR_H
/*
 * Overlay for ip header used by other protocols (tcp, udp).
 */
struct ipovly {
    u_char  ih_x1[9];       /* (unused) */
    u_char  ih_pr;          /* protocol */
    u_short ih_len;         /* protocol length */
    struct  in_addr ih_src;     /* source internet address */
    struct  in_addr ih_dst;     /* destination internet address */
};
#endif /* HAVE_NETINET_IP_VAR_H */

#ifndef HAVE_NETINET_UDP_VAR_H
/* 
 * UDP kernel structures and variables.
 */
struct  udpiphdr {
    struct  ipovly ui_i;        /* overlaid ip structure */
    struct  udphdr ui_u;        /* udp header */
};
#define ui_x1       ui_i.ih_x1
#define ui_pr       ui_i.ih_pr
#define ui_len      ui_i.ih_len
#define ui_src      ui_i.ih_src
#define ui_dst      ui_i.ih_dst
#define ui_sport    ui_u.uh_sport
#define ui_dport    ui_u.uh_dport
#define ui_ulen     ui_u.uh_ulen
#define ui_sum      ui_u.uh_sum

#endif /* HAVE_NETINET_UDP_VAR_H */

#ifndef HAVE_IN_ADDR_T
 typedef unsigned int in_addr_t;
#endif

#ifndef IPVERSION
 #define IPVERSION 4
#endif

#ifndef IP_DF
 #define IP_DF 0x4000
#endif

/* prevent windows error messages from closing DOS window */
#define winpause(x...) do { fprintf(stdout,x); \
                            fflush(stdout); \
                            while ( getchar() != '\n'); \
                          } while (0)
#define winfatal(x...) do { fprintf(stdout,x); \
							fprintf(stdout,"\nPress any key to continue.\n"); \
                            fflush(stdout); \
                            while ( getchar() != '\n'); \
							exit(-1); \
                          } while (0)

/* prettyprint */
#define NIPQUAD(addr) \
  ((unsigned char *)&addr)[0], \
  ((unsigned char *)&addr)[1], \
  ((unsigned char *)&addr)[2], \
  ((unsigned char *)&addr)[3]

/* error messages */
#define WIN_SEND_ERR "** This Windows system blocks raw sockets; reverting to raw ethernet.\n"
#define WINFAIL 999
#define func() do {if (DEBUG) fprintf(stdout,"[%s()][file: %s][line: %d]\n",__FUNCTION__,__FILE__,__LINE__); } while (0)
#define warn(x...) do {fprintf(stdout,"*** Warn: "); fprintf(stdout,x); fprintf(stdout,"\n");} while (0)
#define debug(level,x...) do {if (verbose >= level) {fprintf(stdout,x);} } while (0)

typedef void Sigfunc(int);

/* precasts: net.c */
int craftPacket(unsigned char *, unsigned char *, int, struct sockaddr_in *, struct sockaddr_in *, u_char, unsigned short);
void craftReport(unsigned char *, unsigned short, test_t *);
unsigned short in_cksum(unsigned short *, int);
void genSequence(unsigned char *, int);
Sigfunc *signal(int signo, Sigfunc *func);
Sigfunc *Signal(int signo, Sigfunc *func);
void sig_chld(int signo);
ssize_t Readn(int fd, void *ptr, size_t nbytes, time_t wait);
void Writen(int fd, void *ptr, size_t nbytes);
ssize_t Readline(int fd, void *ptr, size_t maxlen);
ssize_t readn(int fd, void *vptr, size_t n, time_t wait);
int checkRFC1918(struct in_addr *in);
int newSocket(int, int);
int connectSock(int, struct sockaddr_in *, int);
int optSocketTTL(int, int);
int optSocketHDRINCL(int);
int socketSend(int, void *, int, struct sockaddr_in *, int);
int socketSendSpoof(int, void *, int, struct sockaddr_in *, int);

/* precasts: util.c */
unsigned long byte32Swap(unsigned long);
unsigned short byte16Swap(unsigned short);
void binDump(unsigned char *, int);
void checkroot();
void confirm();
void win_init();
void banner();
void banner2();
void spoofSleep();
void fatal(char *, ...);
void pfatal(char *);
void streamSetup();
char *timestamp();

/* precasts: trace.c */
int findtr(char *traceroute);
int runtrace(struct in_addr *dest, char *result, int num);
int checkLinuxTraceroute(char *tracecmd);

/* precasts: dnetspoof.c */
int etherSend();
int registerEther();
int deregisterEther();
int findInterfaces();
void selectInterface();

/* precasts: messages.c */
char *type(uint8_t typ);
void printAddressList(struct in_addr *iplist, int num);
void printTest(test_t *test, int);
void printTestList(unsigned char *msg, int);
void printMsg(unsigned char *, int);
int readMsg(int sock, spoof_msg_t **message, enum msg_types);
void defaultMsg(spoof_msg_t *msg);
int errorRequest(unsigned char *);
int helloRequest(unsigned char *);
int reportRequest(unsigned char *msg, unsigned char *seqno, unsigned long src_addr, unsigned long dst_addr, int nat);
int doneRequest(unsigned char *msg);
int probeRequest(unsigned char *msg);
int readProbeList(unsigned char *buf, test_t **testlistptr);
int traceFilterRequest(unsigned char *msg);

#endif
